USE AdventureWorks2012;
go

-- Check data used in this example
SELECT * FROM Sales.SalesPersonSalesByTerritory
ORDER BY SalesPersonID, TerritoryID;

-- Show sales by territory for each salesperson
SELECT      SalesPersonID, Name, TerritoryID,
            TotalSaleAmt
FROM        Sales.SalesPersonSalesByTerritory
ORDER BY    SalesPersonID, Name, TerritoryID;

-- Show total sales for each salesperson
SELECT      SalesPersonID, Name,
            SUM(TotalSaleAmt) AS TotalSales
FROM        Sales.SalesPersonSalesByTerritory
GROUP BY    SalesPersonID, Name
ORDER BY    SalesPersonID, Name;



-- Show sales by territory as percentage of total sales for each salesperson

-- Before SQL Server 2005, this required either a subquery or a join to a derived table

-- Subquery:
SELECT      s1.SalesPersonID, s1.Name, s1.TerritoryID,
            s1.TotalSaleAmt,
            100 * (s1.TotalSaleAmt / (SELECT SUM(s2.TotalSaleAmt)
                                      FROM   Sales.SalesPersonSalesByTerritory AS s2
                                      WHERE  s2.SalesPersonID = s1.SalesPersonID)) AS PercOfTotal
FROM        Sales.SalesPersonSalesByTerritory AS s1
ORDER BY    s1.SalesPersonID, s1.Name, s1.TerritoryID;

-- Join to derived table:
SELECT      s1.SalesPersonID, s1.Name, s1.TerritoryID,
            s1.TotalSaleAmt,
            100 * (s1.TotalSaleAmt / s2.TotalSales) AS  PercOfTotal
FROM        Sales.SalesPersonSalesByTerritory       AS  s1
INNER JOIN (SELECT      SalesPersonID,
                        SUM(TotalSaleAmt)           AS  TotalSales
            FROM        Sales.SalesPersonSalesByTerritory
            GROUP BY    SalesPersonID)              AS  s2
      ON    s2.SalesPersonID = s1.SalesPersonID
ORDER BY    s1.SalesPersonID, s1.Name, s1.TerritoryID;




-- Since SQL Server 2005, the OVER clause comes to the rescue
-- Here's a quick demo to show the syntax and results:
SELECT      SalesPersonID, Name, TerritoryID,
            TotalSaleAmt,
            SUM(TotalSaleAmt) OVER (PARTITION BY SalesPersonID)
--        , SUM(TotalSaleAmt) OVER ()
FROM        Sales.SalesPersonSalesByTerritory
ORDER BY    SalesPersonID, Name, TerritoryID;





-- And here's the real thing:
SELECT      SalesPersonID, Name, TerritoryID,
            TotalSaleAmt,
            100 * (TotalSaleAmt / SUM(TotalSaleAmt) OVER (PARTITION BY SalesPersonID)) AS PercOfTotal
FROM        Sales.SalesPersonSalesByTerritory
ORDER BY    SalesPersonID, Name, TerritoryID;


-- So what if management wants results as a percentage overall?
-- Simply use an "empty" OVER clause to disable partitioning
-- Here's a quick demo to show the syntax and results:
SELECT      SalesPersonID, Name, TerritoryID,
            TotalSaleAmt,
            SUM(TotalSaleAmt) OVER ()
FROM        Sales.SalesPersonSalesByTerritory
ORDER BY    SalesPersonID, Name, TerritoryID;

-- And here's the real thing:
SELECT      SalesPersonID, Name, TerritoryID,
            TotalSaleAmt,
            100 * (TotalSaleAmt / SUM(TotalSaleAmt) OVER ()) AS PercOfTotal
FROM        Sales.SalesPersonSalesByTerritory
ORDER BY    SalesPersonID, Name, TerritoryID;



-- You can even mix OVER clause and regular GROUP BY aggregates,
-- e.g. to report totals per salesperson as percentage overall:
-- Here's a quick demo to show the syntax and results:
-- This will not work...
SELECT      SalesPersonID, Name,
            SUM(TotalSaleAmt)         AS TotalSales,
            SUM(TotalSaleAmt) OVER () AS GrandTotal
FROM        Sales.SalesPersonSalesByTerritory
GROUP BY    SalesPersonID, Name
ORDER BY    SalesPersonID, Name;

-- This will:
SELECT      SalesPersonID, Name,
            SUM(TotalSaleAmt)              AS TotalSales,
            SUM(SUM(TotalSaleAmt)) OVER () AS GrandTotal
FROM        Sales.SalesPersonSalesByTerritory
GROUP BY    SalesPersonID, Name
ORDER BY    SalesPersonID, Name;

-- And here's the final query:
SELECT      SalesPersonID, Name,
            SUM(TotalSaleAmt)              AS TotalSales,
            100 * SUM(TotalSaleAmt) / SUM(SUM(TotalSaleAmt)) OVER () AS PercOfTotal
FROM        Sales.SalesPersonSalesByTerritory
GROUP BY    SalesPersonID, Name
ORDER BY    SalesPersonID, Name;
