USE AdventureWorks2012;
go

-- Check data used in this example
SELECT * FROM Sales.SalesPersonSalesByDate
ORDER BY SalesPersonID, OrderDate;


-- Add a running total of OrderCount and SaleAmount;
-- order by salesperson / date.

-- Before SQL Server 2005, the best way to do this was a
-- join of each row to all preceding rows - can be very slow!
-- So slow, in fact, that most people would create a temp table
-- and use a cursor or other procedural code to fill it.
-- (That code could be long and complex too - but it's out of scope for this session)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             SUM(s2.OrderCount) AS RunningTotalCount,
             s1.TotalSaleAmt,
             SUM(s2.TotalSaleAmt) AS RunningTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
INNER JOIN   Sales.SalesPersonSalesByDate AS s2
      ON     s2.SalesPersonID <= s1.SalesPersonID
      AND NOT
        (    s2.SalesPersonID = s1.SalesPersonID
         AND s2.OrderDate > s1.OrderDate)
GROUP BY     s1.SalesPersonID, s1.Name, s1.OrderDate, s1.OrderCount, s1.TotalSaleAmt
ORDER BY     s1.SalesPersonID, s1.OrderDate;

-- (And if the ordering column(s) are not unique in the set,
--  the query would have been even more complex...)




-- In SQL Server 2005, the OVER clause did not help for this problem.
-- But the APPLY clause did help (a bit) - at the cost of performance and ANSI compliance.
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             d1.RunningTotalCount,
             s1.TotalSaleAmt,
             d1.RunningTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
CROSS APPLY (SELECT SUM(s2.OrderCount) AS RunningTotalCount,
                    SUM(s2.TotalSaleAmt) AS RunningTotalSales
             FROM   Sales.SalesPersonSalesByDate AS s2
             WHERE  s2.SalesPersonID <= s1.SalesPersonID
             AND NOT
               (    s2.SalesPersonID = s1.SalesPersonID
                AND s2.OrderDate > s1.OrderDate)) AS d1
ORDER BY     s1.SalesPersonID, s1.OrderDate;






-- But SQL Server 2012 really simplifies things!
-- (and all ANSI standard!)
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             SUM(OrderCount)   OVER (ORDER BY SalesPersonID, OrderDate
                                     ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS RunningTotalCount,
             TotalSaleAmt,
             SUM(TotalSaleAmt) OVER (ORDER BY SalesPersonID, OrderDate
                                     ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS RunningTotalSales
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;


-- This can be shortened in two ways:
-- 1. Use ROWS UNBOUNDED PRECEDING as shorthand (because CURRENT ROW is default end specification)
-- 2. Use only the ORDER BY clause (because UNBOUNDED PRECEDING is the default start specification)
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             SUM(OrderCount)   OVER (ORDER BY SalesPersonID, OrderDate
                                     ROWS UNBOUNDED PRECEDING) AS RunningTotalCount,
             TotalSaleAmt,
             SUM(TotalSaleAmt) OVER (ORDER BY SalesPersonID, OrderDate) AS RunningTotalSales
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;
