USE AdventureWorks2012;
go

-- Slightly different problem - multiple running totals with DIFFERENT specification

-- We need two running totals of SaleAmount,
-- one that resets for each sales person, and one overall.

-- The "self-join" used before will not work;
-- we would need two self-joins, but then we would get many extra rows
-- as a result of the cartesian product between the first and second self-join.

-- The only way to do this in a single query requires correlated subqueries.
-- Especially on large tables, this can be very slow!
-- (You could replace one of the subqueries with a self-join,
--  but that would neither simplify nor speed up the query).
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             s1.TotalSaleAmt,
            (SELECT SUM(s2.TotalSaleAmt)
             FROM   Sales.SalesPersonSalesByDate AS s2
             WHERE  s2.SalesPersonID = s1.SalesPersonID
             AND    s2.OrderDate <= s1.OrderDate) AS RunningBySalesPerson,
            (SELECT SUM(s3.TotalSaleAmt)
             FROM   Sales.SalesPersonSalesByDate AS s3
             WHERE  s3.SalesPersonID <= s1.SalesPersonID
             AND NOT
               (    s3.SalesPersonID = s1.SalesPersonID
                AND s3.OrderDate > s1.OrderDate)) AS RunningOverall
FROM         Sales.SalesPersonSalesByDate AS s1
ORDER BY     s1.SalesPersonID, s1.OrderDate;

-- (And if the ordering column(s) are not unique in the set,
--  the query would have been even more complex...)





-- In SQL Server 2005, the APPLY version can be used for different running totals,
-- because the aggregation is done in the subquery, before the join.
-- (In this case, the performance is roughly the same as the previous version).
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             s1.TotalSaleAmt,
             d1.RunningBySalesPerson,
             d2.RunningOverall
FROM         Sales.SalesPersonSalesByDate AS s1
CROSS APPLY (SELECT SUM(s2.TotalSaleAmt) AS RunningBySalesPerson
             FROM   Sales.SalesPersonSalesByDate AS s2
             WHERE  s2.SalesPersonID = s1.SalesPersonID
             AND    s2.OrderDate <= s1.OrderDate) AS d1
CROSS APPLY (SELECT SUM(s3.TotalSaleAmt) AS RunningOverall
             FROM   Sales.SalesPersonSalesByDate AS s3
             WHERE  s3.SalesPersonID <= s1.SalesPersonID
             AND NOT
               (    s3.SalesPersonID = s1.SalesPersonID
                AND s3.OrderDate > s1.OrderDate)) AS d2
ORDER BY     s1.SalesPersonID, s1.OrderDate;






-- In SQL Server 2012, it is again simple (and fast).
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             TotalSaleAmt,
             SUM(TotalSaleAmt) OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS RunningBySalesPerson,
             SUM(TotalSaleAmt) OVER (ORDER BY SalesPersonID, OrderDate
                                     ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS RunningOverall
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;


-- (Or the shortened version)
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             TotalSaleAmt,
             SUM(TotalSaleAmt) OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate) AS RunningBySalesPerson,
             SUM(TotalSaleAmt) OVER (ORDER BY SalesPersonID, OrderDate) AS RunningOverall
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;
