USE AdventureWorks2012;
go

-- Check data used in this example
SELECT * FROM Sales.SalesPersonSalesByDate
ORDER BY SalesPersonID, OrderDate;


-- Add the average of current and four previous SaleAmount and OrderCount values;
-- order by date and restart for each salesperson.
-- If we knew that every date is in the table, we could use DATEDIFF ... but alas!

-- Before SQL Server 2005, we could use the same
-- self-join technique as for running totals,
-- but with a subquery to limit the number of rows:
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             AVG(CAST(s2.OrderCount AS decimal(3,0))) AS AvgOrderCount,
             s1.TotalSaleAmt,
             AVG(s2.TotalSaleAmt) AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
INNER JOIN   Sales.SalesPersonSalesByDate AS s2
      ON     s2.SalesPersonID = s1.SalesPersonID
      AND    s2.OrderDate <= s1.OrderDate
      AND   (SELECT COUNT(*)
             FROM   Sales.SalesPersonSalesByDate AS s3
             WHERE  s3.SalesPersonID = s2.SalesPersonID
             AND    s3.OrderDate BETWEEN s2.OrderDate AND s1.OrderDate) <= 5         -- Current row + 4 previous = 5
GROUP BY     s1.SalesPersonID, s1.Name, s1.OrderDate, s1.OrderCount, s1.TotalSaleAmt
ORDER BY     s1.SalesPersonID, s1.OrderDate;

-- (And, again, this would get nasty with non-unique criteria)






-- In SQL Server 2005, this problem could be tackled more easily
-- by using the ROW_NUMBER function (with OVER clause!) to provide gapless numbering,
-- combined with a CTE (common table expression - also new in SQL Server 2005).
WITH NumberedRows
AS (SELECT   SalesPersonID, Name, OrderDate, OrderCount, TotalSaleAmt,
             ROW_NUMBER() OVER (PARTITION BY SalesPersonID
                                ORDER BY OrderDate) AS rn
    FROM     Sales.SalesPersonSalesByDate)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             AVG(CAST(s2.OrderCount AS decimal(3,0))) AS AvgOrderCount,
             s1.TotalSaleAmt,
             AVG(s2.TotalSaleAmt) AS AvgTotalSales
FROM         NumberedRows AS s1
INNER JOIN   NumberedRows AS s2
      ON     s2.SalesPersonID = s1.SalesPersonID
      AND    s2.rn BETWEEN s1.rn - 4 AND s1.rn
GROUP BY     s1.SalesPersonID, s1.Name, s1.OrderDate, s1.OrderCount, s1.TotalSaleAmt
ORDER BY     s1.SalesPersonID, s1.OrderDate;

-- However, for better performance you would forego the ROW_NUMBER,
-- and instead use a combination of TOP and APPLY - both not ANSI standard, though
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             AVG(CAST(d1.OrderCount AS decimal(3,0))) AS AvgOrderCount,
             s1.TotalSaleAmt,
             AVG(d1.TotalSaleAmt) AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
CROSS APPLY (SELECT TOP(5)               -- Current row + 4 previous = 5
                      s2.OrderCount, s2.TotalSaleAmt
             FROM     Sales.SalesPersonSalesByDate AS s2
             WHERE    s2.SalesPersonID = s1.SalesPersonID
             AND      s2.OrderDate <= s1.OrderDate
             ORDER BY s2.OrderDate DESC) AS d1
GROUP BY     s1.SalesPersonID, s1.Name, s1.OrderDate, s1.OrderCount, s1.TotalSaleAmt
ORDER BY     s1.SalesPersonID, s1.OrderDate;




-- But SQL Server 2012 really simplifies things!
-- (And, again, fully ANSI standard)
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             AVG(CAST(OrderCount AS decimal(3,0)))
                               OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS BETWEEN 4 PRECEDING AND CURRENT ROW) AS AvgOrderCount,
             TotalSaleAmt,
             AVG(TotalSaleAmt) OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS BETWEEN 4 PRECEDING AND CURRENT ROW) AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;


-- Shorter version (just syntactical suger)
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             AVG(CAST(OrderCount AS decimal(3,0)))
                               OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS 4 PRECEDING) AS AvgOrderCount,
             TotalSaleAmt,
             AVG(TotalSaleAmt) OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS 4 PRECEDING) AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;
