USE AdventureWorks2012;
go

-- Just as with running totals, the pre-SQL 2005 self-join
-- can not be used for multiple moving aggregates with different specification.
-- Or for a combination of moving aggregates and running totals.

-- So let's say we need to look back four rows for SaleAmount, but three for OrderCount.
-- We now need correlated subqueries for each aggregate,
-- with a NESTED(!!) correlated subquery to find the last row to include.
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
            (SELECT AVG(CAST(s2.OrderCount AS decimal(3,0)))
             FROM   Sales.SalesPersonSalesByDate AS s2
             WHERE  s2.SalesPersonID = s1.SalesPersonID
             AND    s2.OrderDate <= s1.OrderDate
             AND   (SELECT COUNT(*)
                    FROM   Sales.SalesPersonSalesByDate AS s3
                    WHERE  s3.SalesPersonID = s2.SalesPersonID
                    AND    s3.OrderDate BETWEEN s2.OrderDate
                                            AND s1.OrderDate) <= 4)
                                                            AS AvgOrderCount,
             s1.TotalSaleAmt,
            (SELECT AVG(s4.TotalSaleAmt)
             FROM   Sales.SalesPersonSalesByDate AS s4
             WHERE  s4.SalesPersonID = s1.SalesPersonID
             AND    s4.OrderDate <= s1.OrderDate
             AND   (SELECT COUNT(*)
                    FROM   Sales.SalesPersonSalesByDate AS s5
                    WHERE  s5.SalesPersonID = s4.SalesPersonID
                    AND    s5.OrderDate BETWEEN s4.OrderDate
                                            AND s1.OrderDate) <= 5)
                                                            AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
ORDER BY     s1.SalesPersonID, s1.OrderDate;

-- (And if the ordering column(s) are not unique in the set,
--  the query would have been even more complex...)

-- If ANSI compliance is not an issue, you can instead use the TOP operator
-- (query inspired by Peter Larsson)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
            (SELECT AVG(CAST(d1.OrderCount AS decimal(3,0)))
             FROM  (SELECT TOP(4)
                             s2.OrderCount
                    FROM     Sales.SalesPersonSalesByDate AS s2
                    WHERE    s2.SalesPersonID = s1.SalesPersonID
                    AND      s2.OrderDate <= s1.OrderDate
                    ORDER BY s2.OrderDate DESC) AS d1) AS AvgOrderCount,
            s1.TotalSaleAmt,
            (SELECT AVG(d2.TotalSaleAmt)
             FROM  (SELECT TOP(5)
                             s3.TotalSaleAmt
                    FROM     Sales.SalesPersonSalesByDate AS s3
                    WHERE    s3.SalesPersonID = s1.SalesPersonID
                    AND      s3.OrderDate <= s1.OrderDate
                    ORDER BY s3.OrderDate DESC) AS d2) AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
ORDER BY     s1.SalesPersonID, s1.OrderDate;






-- The SQL Server 2005 version still uses ROW_NUMBER.
-- Optionally, the subqueries can be replaced by APPLY (but not JOIN!)
-- (Unfortunately, both versions have terrible performance)
WITH NumberedRows
AS (SELECT   SalesPersonID, Name, OrderDate, OrderCount, TotalSaleAmt,
             ROW_NUMBER() OVER (PARTITION BY SalesPersonID
                                ORDER BY OrderDate) AS rn
    FROM     Sales.SalesPersonSalesByDate)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
            (SELECT AVG(CAST(s2.OrderCount AS decimal(3,0)))
             FROM   NumberedRows AS s2
             WHERE  s2.SalesPersonID = s1.SalesPersonID
             AND    s2.rn BETWEEN s1.rn - 3 AND s1.rn) AS AvgOrderCount,
             s1.TotalSaleAmt,
            (SELECT AVG(s3.TotalSaleAmt)
             FROM   NumberedRows AS s3
             WHERE  s3.SalesPersonID = s1.SalesPersonID
             AND    s3.rn BETWEEN s1.rn - 4 AND s1.rn) AS AvgTotalSales
FROM         NumberedRows AS s1
ORDER BY     s1.SalesPersonID, s1.OrderDate;

WITH NumberedRows
AS (SELECT   SalesPersonID, Name, OrderDate, OrderCount, TotalSaleAmt,
             ROW_NUMBER() OVER (PARTITION BY SalesPersonID
                                ORDER BY OrderDate) AS rn
    FROM     Sales.SalesPersonSalesByDate)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             d1.AvgOrderCount,
             s1.TotalSaleAmt,
             d2.AvgTotalSales
FROM         NumberedRows AS s1
CROSS APPLY (SELECT AVG(CAST(s2.OrderCount AS decimal(3,0))) AS AvgOrderCount
             FROM   NumberedRows AS s2
             WHERE  s2.SalesPersonID = s1.SalesPersonID
             AND    s2.rn BETWEEN s1.rn - 3 AND s1.rn) AS d1
CROSS APPLY (SELECT AVG(s2.TotalSaleAmt) AS AvgTotalSales
             FROM   NumberedRows AS s2
             WHERE  s2.SalesPersonID = s1.SalesPersonID
             AND    s2.rn BETWEEN s1.rn - 4 AND s1.rn) AS d2
ORDER BY     s1.SalesPersonID, s1.OrderDate;

-- The fastest alternative for SQL 2005 doesn't use ROW_NUMBER at all,
-- but does use a combination of TOP and APPLY.
-- This time, we have to APPLY multiple subqueries,
-- and aggregate within them - which requires an extra nesting level.
-- (query inspired by Peter Larsson)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             d2.AvgOrderCount,
             s1.TotalSaleAmt,
             d4.AvgTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
CROSS APPLY (SELECT   AVG(CAST(d1.OrderCount AS decimal(3,0))) AS AvgOrderCount
             FROM    (SELECT TOP(4)
                               s2.OrderCount
                      FROM     Sales.SalesPersonSalesByDate AS s2
                      WHERE    s2.SalesPersonID = s1.SalesPersonID
                      AND      s2.OrderDate <= s1.OrderDate
                      ORDER BY s2.OrderDate DESC) AS d1) AS d2
CROSS APPLY (SELECT   AVG(d3.TotalSaleAmt) AS AvgTotalSales
             FROM    (SELECT TOP(5)
                               s3.TotalSaleAmt
                      FROM     Sales.SalesPersonSalesByDate AS s3
                      WHERE    s3.SalesPersonID = s1.SalesPersonID
                      AND      s3.OrderDate <= s1.OrderDate
                      ORDER BY s3.OrderDate DESC) AS d3) AS d4
ORDER BY     s1.SalesPersonID, s1.OrderDate;






-- The SQL Server 2012 version is, again, easy, fast, and ANSI standard
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             AVG(CAST(OrderCount AS decimal(3,0)))
                               OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS BETWEEN 3 PRECEDING AND CURRENT ROW) AS AvgOrderCount,
             TotalSaleAmt,
             AVG(TotalSaleAmt) OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS BETWEEN 4 PRECEDING AND CURRENT ROW) AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;


-- Shorter version (just syntactical suger)
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             AVG(CAST(OrderCount AS decimal(3,0)))
                               OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS 3 PRECEDING) AS AvgOrderCount,
             TotalSaleAmt,
             AVG(TotalSaleAmt) OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS 4 PRECEDING) AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;
