USE AdventureWorks2012;
go



-- Traditional moving average always lags behind.
-- You can solve that by using the average of a group of rows
-- that is centered around the "current" row.

-- For instance:
-- * average of current, two previous, and two next rows for OrderCount
-- * average of current, three previous, and three next rows for SaleAmount

-- Very simple in SQL Server 2012
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             AVG(CAST(OrderCount AS decimal(3,0)))
                               OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS BETWEEN 2 PRECEDING AND 2 FOLLOWING) AS AvgOrderCount,
             TotalSaleAmt,
             AVG(TotalSaleAmt) OVER (PARTITION BY SalesPersonID
                                     ORDER BY OrderDate
                                     ROWS BETWEEN 3 PRECEDING AND 3 FOLLOWING) AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;



-- The "slow" version for SQL Server 2005 is almost unchanged
WITH NumberedRows
AS (SELECT   SalesPersonID, Name, OrderDate, OrderCount, TotalSaleAmt,
             ROW_NUMBER() OVER (PARTITION BY SalesPersonID
                                ORDER BY OrderDate) AS rn
    FROM     Sales.SalesPersonSalesByDate)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             d1.AvgOrderCount,
             s1.TotalSaleAmt,
             d2.AvgTotalSales
FROM         NumberedRows AS s1
CROSS APPLY (SELECT AVG(CAST(s2.OrderCount AS decimal(3,0))) AS AvgOrderCount
             FROM   NumberedRows AS s2
             WHERE  s2.SalesPersonID = s1.SalesPersonID
             AND    s2.rn BETWEEN s1.rn - 2 AND s1.rn + 2) AS d1
CROSS APPLY (SELECT AVG(s3.TotalSaleAmt) AS AvgTotalSales
             FROM   NumberedRows AS s3
             WHERE  s3.SalesPersonID = s1.SalesPersonID
             AND    s3.rn BETWEEN s1.rn - 3 AND s1.rn + 3) AS d2
ORDER BY     s1.SalesPersonID, s1.OrderDate;

-- The "fast" version for SQL Server 2005 becomes very ugly now!
-- (query inspired by Peter Larsson)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             d4.AvgOrderCount,
             s1.TotalSaleAmt,
             d8.AvgTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
CROSS APPLY (SELECT AVG(CAST(d3.OrderCount AS decimal(3,0))) AS AvgOrderCount
             FROM  (SELECT   d1.OrderCount
                    FROM    (SELECT TOP(3)    -- Current and previous 2
                                      s2.OrderCount
                             FROM     Sales.SalesPersonSalesByDate AS s2
                             WHERE    s2.SalesPersonID = s1.SalesPersonID
                             AND      s2.OrderDate <= s1.OrderDate  -- Includes current
                             ORDER BY s2.OrderDate DESC) AS d1
                    UNION ALL
                    SELECT   d2.OrderCount
                    FROM    (SELECT TOP(2)    -- Next 2
                                      s3.OrderCount
                             FROM     Sales.SalesPersonSalesByDate AS s3
                             WHERE    s3.SalesPersonID = s1.SalesPersonID
                             AND      s3.OrderDate > s1.OrderDate  -- Excludes current
                             ORDER BY s3.OrderDate ASC) AS d2) AS d3) AS d4
CROSS APPLY (SELECT AVG(d7.TotalSaleAmt) AS AvgTotalSales
             FROM  (SELECT   d5.TotalSaleAmt
                    FROM    (SELECT TOP(4)    -- Current and previous 3
                                      s4.TotalSaleAmt
                             FROM     Sales.SalesPersonSalesByDate AS s4
                             WHERE    s4.SalesPersonID = s1.SalesPersonID
                             AND      s4.OrderDate <= s1.OrderDate  -- Includes current
                             ORDER BY s4.OrderDate DESC) AS d5
                    UNION ALL
                    SELECT   d6.TotalSaleAmt
                    FROM    (SELECT TOP(3)    -- Next 3
                                      s5.TotalSaleAmt
                             FROM     Sales.SalesPersonSalesByDate AS s5
                             WHERE    s5.SalesPersonID = s1.SalesPersonID
                             AND      s5.OrderDate > s1.OrderDate  -- Excludes current
                             ORDER BY s5.OrderDate ASC) AS d6) AS d7) AS d8
ORDER BY     s1.SalesPersonID, s1.OrderDate;



-- The SQL Server 2000 code is only slightly worse than the "fast" SQL 2005 version
-- (query by Peter Larsson)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
            (SELECT AVG(CAST(d3.OrderCount AS decimal(3,0)))
             FROM  (SELECT d1.OrderCount
                    FROM  (SELECT TOP(3)    -- Current and previous 2
                                    s2.OrderCount
                           FROM     Sales.SalesPersonSalesByDate AS s2
                           WHERE    s2.SalesPersonID = s1.SalesPersonID
                           AND      s2.OrderDate <= s1.OrderDate  -- Includes current
                           ORDER BY s2.OrderDate DESC) AS d1
                    UNION ALL
                    SELECT d2.OrderCount
                    FROM  (SELECT TOP(2)    -- Next 2
                                    s3.OrderCount
                           FROM     Sales.SalesPersonSalesByDate AS s3
                           WHERE    s3.SalesPersonID = s1.SalesPersonID
                           AND      s3.OrderDate > s1.OrderDate  -- Excludes current
                           ORDER BY s3.OrderDate ASC) AS d2) AS d3) AS AvgOrderCount,
             s1.TotalSaleAmt,
            (SELECT AVG(d6.TotalSaleAmt)
             FROM  (SELECT d4.TotalSaleAmt
                    FROM  (SELECT TOP(4)    -- Current and previous 3
                                    s4.TotalSaleAmt
                           FROM     Sales.SalesPersonSalesByDate AS s4
                           WHERE    s4.SalesPersonID = s1.SalesPersonID
                           AND      s4.OrderDate <= s1.OrderDate  -- Includes current
                           ORDER BY s4.OrderDate DESC) AS d4
                    UNION ALL
                    SELECT d5.TotalSaleAmt
                    FROM  (SELECT TOP(3)    -- Next 3
                                    s5.TotalSaleAmt
                           FROM     Sales.SalesPersonSalesByDate AS s5
                           WHERE    s5.SalesPersonID = s1.SalesPersonID
                           AND      s5.OrderDate > s1.OrderDate  -- Excludes current
                           ORDER BY s5.OrderDate ASC) AS d5) AS d6) AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
ORDER BY     s1.SalesPersonID, s1.OrderDate;


-- And finally, an ANSI-compliant version for SQL 2000.
-- The code is a lot cleaner, but it is slower than using TOP
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
            (SELECT AVG(CAST(s2.OrderCount AS decimal(3,0)))
             FROM   Sales.SalesPersonSalesByDate AS s2
             WHERE  s2.SalesPersonID = s1.SalesPersonID
             AND   (SELECT COUNT(*)
                    FROM   Sales.SalesPersonSalesByDate AS s3
                    WHERE  s3.SalesPersonID = s2.SalesPersonID
                    AND    s3.OrderDate BETWEEN s2.OrderDate
                                            AND s1.OrderDate) <= 3   -- Current row + 2 previous = 3
             AND   (SELECT COUNT(*)
                    FROM   Sales.SalesPersonSalesByDate AS s4
                    WHERE  s4.SalesPersonID = s2.SalesPersonID
                    AND    s4.OrderDate BETWEEN s1.OrderDate
                                            AND s2.OrderDate) <= 3)  -- Current row + 2 next = 3
                                                            AS AvgTotalSales,
             s1.TotalSaleAmt,
            (SELECT AVG(s5.TotalSaleAmt)
             FROM   Sales.SalesPersonSalesByDate AS s5
             WHERE  s5.SalesPersonID = s1.SalesPersonID
             AND   (SELECT COUNT(*)
                    FROM   Sales.SalesPersonSalesByDate AS s6
                    WHERE  s6.SalesPersonID = s5.SalesPersonID
                    AND    s6.OrderDate BETWEEN s5.OrderDate
                                            AND s1.OrderDate) <= 4   -- Current row + 3 previous = 4
             AND   (SELECT COUNT(*)
                    FROM   Sales.SalesPersonSalesByDate AS s7
                    WHERE  s7.SalesPersonID = s5.SalesPersonID
                    AND    s7.OrderDate BETWEEN s1.OrderDate
                                            AND s5.OrderDate) <= 4)  -- Current row + 3 next = 4
                                                            AS AvgTotalSales
FROM         Sales.SalesPersonSalesByDate AS s1
ORDER BY     s1.SalesPersonID, s1.OrderDate;
