USE AdventureWorks2012;
go

-- Check data used in this example
SELECT * FROM Sales.SalesPersonSalesByDate
ORDER BY SalesPersonID, OrderDate;


-- Report now needs to include the difference in OrderCount and TotalSaleAmt
-- since the "previous" OrderDate for the same salesperson.

-- In SQL Server 2000, solving this with a join is hard,
-- because we then would need to add nested subqueries
-- to make sure we join to "the", not "a" previous row.
-- Better to use a subquery with TOP.
-- (Unfortunately, this subquery needs to be repeated)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             s1.OrderCount - (SELECT TOP(1) s2.OrderCount
                              FROM          Sales.SalesPersonSalesByDate AS s2
                              WHERE         s2.SalesPersonID = s1.SalesPersonID
                              AND           s2.OrderDate < s1.OrderDate
                              ORDER BY      s2.OrderDate DESC) AS DeltaCount,
             s1.TotalSaleAmt,
             s1.TotalSaleAmt - (SELECT TOP(1) s3.TotalSaleAmt
                                FROM          Sales.SalesPersonSalesByDate AS s3
                                WHERE         s3.SalesPersonID = s1.SalesPersonID
                                AND           s3.OrderDate < s1.OrderDate
                                ORDER BY      s3.OrderDate DESC) AS DeltaSale
FROM         Sales.SalesPersonSalesByDate AS s1
ORDER BY     s1.SalesPersonID, s1.OrderDate;




-- In SQL Server 2005, you have two options for using a join.
-- Option 1: Use ROW_NUMBER() to get consecutive numbers
WITH NumberedRows
AS (SELECT   SalesPersonID, Name, OrderDate, OrderCount, TotalSaleAmt,
             ROW_NUMBER() OVER (PARTITION BY SalesPersonID
                                ORDER BY OrderDate) AS rn
    FROM     Sales.SalesPersonSalesByDate)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             s1.OrderCount - s2.OrderCount AS DeltaCount,
             s1.TotalSaleAmt,
             s1.TotalSaleAmt - s2.TotalSaleAmt AS DeltaSale
FROM         NumberedRows AS s1
LEFT  JOIN   NumberedRows AS s2
      ON     s2.SalesPersonID = s1.SalesPersonID
      AND    s2.rn = s1.rn - 1
ORDER BY     s1.SalesPersonID, s1.OrderDate;

-- Option 2: Use APPLY to simulate a fake join (slower, in this case!)
SELECT       s1.SalesPersonID, s1.Name, s1.OrderDate,
             s1.OrderCount,
             s1.OrderCount - d1.OrderCount AS DeltaCount,
             s1.TotalSaleAmt,
             s1.TotalSaleAmt - d1.TotalSaleAmt AS DeltaSale
FROM         Sales.SalesPersonSalesByDate AS s1
OUTER APPLY (SELECT TOP(1) s2.OrderCount, s2.TotalSaleAmt
             FROM          Sales.SalesPersonSalesByDate AS s2
             WHERE         s2.SalesPersonID = s1.SalesPersonID
             AND           s2.OrderDate < s1.OrderDate
             ORDER BY      s2.OrderDate DESC) AS d1
ORDER BY     s1.SalesPersonID, s1.OrderDate;




-- For SQL Server 2012, we can use the new LAG function
SELECT       SalesPersonID, Name, OrderDate,
             OrderCount,
             OrderCount - LAG(OrderCount) OVER(PARTITION BY SalesPersonID
                                               ORDER BY OrderDate) AS DeltaCount,
             TotalSaleAmt,
             TotalSaleAmt - LAG(TotalSaleAmt) OVER(PARTITION BY SalesPersonID
                                                   ORDER BY OrderDate) AS DeltaSale
FROM         Sales.SalesPersonSalesByDate
ORDER BY     SalesPersonID, OrderDate;
